/*
 * Copyright 2015 Luca Beltrame <lbeltrame@kde.org>
 *
 * This file is part of Danbooru Client.
 *
 * Danbooru Client is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Danbooru Client is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Danbooru Client. If not, see <http://www.gnu.org/licenses/>.
 */

#include <KStatusBar>

#include <QAction>
#include <QQuickWidget>
#include <QQmlContext>
#include <QApplication>
#include <QStandardPaths>
#include <QUrl>

#include <kactioncollection.h>
#include <KStandardAction>
#include <KLocalizedString>
#include <kdeclarative.h>

#include "libdanbooru/danbooruservice.h"
#include "libdanbooru/danboorupost.h"
#include "model/danboorupostmodel.h"
#include "mainwindow.h"

namespace Danbooru
{

DanbooruMainWindow::DanbooruMainWindow(QWidget *parent)
    : KXmlGuiWindow(parent),
      m_view(0),
      m_model(0),
      m_service(0)
{
    auto *m_view = new QQuickWidget(this);
    // tell the KXmlGuiWindow that this is indeed the main widget
    auto *m_model = new DanbooruPostModel(this);
    setCentralWidget(m_view);

    qmlRegisterType<Danbooru::DanbooruPost>("DanbooruClient", 1, 0, "DanbooruPost");
    qmlRegisterType<Danbooru::DanbooruService>("DanbooruClient", 1, 0, "DanbooruService");
    qRegisterMetaType<DanbooruPost::Rating>();

    auto qmlViewPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
                                              qApp->applicationName() + QChar('/') +                                             QLatin1Literal("danbooruimageview.qml"));

    QQmlContext *ctxt = m_view->rootContext();
    ctxt->setContextProperty("danbooruModel", m_model);
    ctxt->setContextProperty("danbooruService", m_service);
    m_view->setSource(QUrl::fromLocalFile(qmlViewPath));

    // then, setup our actions
    setupActions();

    // add a status bar
    statusBar()->show();

    setupGUI(KXmlGuiWindow::ToolBar|Keys|Save|Create|StatusBar, "danbooru-clientui.rc");

}

DanbooruMainWindow::~DanbooruMainWindow()
{
}

void DanbooruMainWindow::setupActions()
{

    QAction *connectAction = new QAction(
        QIcon::fromTheme(QLatin1String("document-open-remote")),
        i18n("Connect..."),
        this);

    QAction *fetchAction = new QAction(QIcon::fromTheme(QLatin1String("download")),
                                       i18n("Download"), this);

    fetchAction->setEnabled(false);

    actionCollection()->addAction(QLatin1String("connect"), connectAction);
    actionCollection()->addAction(QLatin1String("fetch"), fetchAction);
    actionCollection()->setDefaultShortcut(connectAction, KStandardShortcut::Open);
    actionCollection()->setDefaultShortcut(fetchAction, KStandardShortcut::Find);
    // actionCollection()->removeAction(actionCollection()->action("help_contents"));

    KStandardAction::quit(qApp, SLOT(close()), actionCollection());
    connect(connectAction, &QAction::triggered, this, &DanbooruMainWindow::connectToBoard);
    connect(fetchAction, &QAction::triggered, this, &DanbooruMainWindow::downloadPosts);

}

void DanbooruMainWindow::connectToBoard()
{

    if (!m_view) {
        return;
    }

}

void DanbooruMainWindow::downloadPosts()
{
    if (!m_service) {
        return;
    }

    m_service->getPostList(1, QStringList(), 10);

}

void DanbooruMainWindow::optionsPreferences()
{

}

} // namespace Danbooru
